'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _objectHash = require('object-hash');

var _objectHash2 = _interopRequireDefault(_objectHash);

require('isomorphic-fetch');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var string = function string(values) {
  return _lodash2.default.map(values, function (value, key) {
    return encodeURIComponent(key) + '=' + encodeURIComponent(value);
  }).join('&');
};

var normalize = function normalize(_uri, params) {
  var uri = _uri;
  (0, _keys2.default)(params).forEach(function (key) {
    if (uri.match(':' + key)) {
      uri = uri.replace(new RegExp(':' + key, 'g'), encodeURIComponent(params[key]));
      delete params[key];
    }
  });
  return uri;
};

var getHeader = function getHeader(_headers, defaultHeaders, mode, credentials) {
  var headers = (0, _assign2.default)((0, _extends3.default)({}, defaultHeaders), _headers);

  return {
    method: 'GET',
    headers: headers,
    mode: mode,
    credentials: credentials
  };
};

var postHeader = function postHeader(values, _headers, defaultHeaders, mode, credentials, useQuery) {
  var headers = (0, _assign2.default)((0, _extends3.default)({
    'Content-Type': 'application/json'
  }, defaultHeaders), _headers);

  return {
    method: 'POST',
    headers: headers,
    body: !useQuery ? (0, _stringify2.default)(values) : '',
    mode: mode,
    credentials: credentials
  };
};

var patchHeader = function patchHeader(values, _headers, defaultHeaders, mode, credentials, useQuery) {
  var headers = (0, _assign2.default)((0, _extends3.default)({
    'Content-Type': 'application/json'
  }, defaultHeaders), _headers);

  return {
    method: 'PATCH',
    headers: headers,
    body: !useQuery ? (0, _stringify2.default)(values) : '',
    mode: mode,
    credentials: credentials
  };
};

var deleteHeader = function deleteHeader(values, _headers, defaultHeaders, mode, credentials, useQuery) {
  var headers = (0, _assign2.default)((0, _extends3.default)({
    'Content-Type': 'application/json'
  }, defaultHeaders), _headers);

  return {
    method: 'DELETE',
    headers: headers,
    body: !useQuery ? (0, _stringify2.default)(values) : '',
    mode: mode,
    credentials: credentials
  };
};

var ApiClient = function ApiClient() {
  var defaultHeaders = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var logger = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function () {
    var _console;

    return (_console = console).log.apply(_console, arguments);
  };
  (0, _classCallCheck3.default)(this, ApiClient);

  var cached = {};
  this.fetchJSON = function (_ref) {
    var _ref$url = _ref.url,
        url = _ref$url === undefined ? '' : _ref$url,
        _ref$method = _ref.method,
        method = _ref$method === undefined ? 'GET' : _ref$method,
        _ref$values = _ref.values,
        values = _ref$values === undefined ? {} : _ref$values,
        headers = _ref.headers,
        mode = _ref.mode,
        credentials = _ref.credentials,
        cache = _ref.cache,
        useQuery = _ref.useQuery;


    if (!url) {
      throw new Error('URL does not specified');
    }

    var _values = (0, _assign2.default)({}, values);
    var _url = normalize(url, _values) + (useQuery || method === 'GET' ? '?' + string(_values) : '');

    // return from cache
    var hashed = (0, _objectHash2.default)(_url);
    if (cache && method === 'GET' && cached[hashed]) {
      logger('## return from cache ', _url, method, _values);
      return _promise2.default.resolve(cached[hashed]);
    }

    return new _promise2.default(function (resolve, reject) {
      logger('## fetch ', _url, method, _values);

      fetch(_url, method === 'GET' ? getHeader(headers, defaultHeaders, mode, credentials) : method === 'POST' ? postHeader(_values, headers, defaultHeaders, mode, credentials, useQuery) : method === 'PATCH' ? patchHeader(_values, headers, defaultHeaders, mode, credentials, useQuery) : method === 'DELETE' ? deleteHeader(_values, headers, defaultHeaders, mode, credentials, useQuery) : '').then(function (res) {
        if (!res.ok) {
          res.json().then(function (json) {
            reject({
              body: json,
              res: res
            });
          }, function () {
            reject({
              body: {},
              res: res
            });
          });
        } else if (method === 'GET') {
          res.json().then(function (json) {
            if (cache) {
              cached[hashed] = {
                body: _lodash2.default.cloneDeep(json),
                res: _lodash2.default.cloneDeep(res)
              };
            }
            resolve({
              body: json,
              res: res
            });
          }, function () {
            reject({
              body: {},
              res: res
            });
          });
        } else {
          res.json().then(function (json) {
            resolve({
              body: json,
              res: res
            });
          }, function () {
            resolve({
              body: {},
              res: res
            });
          });
        }
      }, function (err) {
        reject({
          body: {},
          err: err
        });
      });
    });
  };
};

exports.default = ApiClient;
module.exports = exports['default'];