'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _passport = require('passport');

var _passport2 = _interopRequireDefault(_passport);

var _passportGithub = require('passport-github2');

var _passportFacebook = require('passport-facebook');

var _passportTwitter = require('passport-twitter');

var _bodyParser = require('body-parser');

var _bodyParser2 = _interopRequireDefault(_bodyParser);

var _cookieParser = require('cookie-parser');

var _cookieParser2 = _interopRequireDefault(_cookieParser);

var _expressSession = require('express-session');

var _expressSession2 = _interopRequireDefault(_expressSession);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var strategies = {
  github: _passportGithub.Strategy,
  facebook: _passportFacebook.Strategy,
  twitter: _passportTwitter.Strategy
};

var applyStrategy = function applyStrategy(authenticator, config, Strategy, origin) {
  _passport2.default.use(new Strategy({
    clientID: config.appId,
    clientSecret: config.secret,
    callbackURL: origin + '/auth/' + authenticator + '/callback'
  }, function (accessToken, refreshToken, profile, cb) {
    return cb(null, (0, _extends3.default)({}, profile, { token: accessToken }));
  }));
};

var applyEndpoint = function applyEndpoint(app, authenticator) {
  app.get('/auth/' + authenticator, _passport2.default.authenticate(authenticator, { session: true }));

  app.get('/auth/' + authenticator + '/callback', _passport2.default.authenticate(authenticator, { session: true, failureRedirect: '/auth/' + authenticator }), function (req, res) {
    var redirect = req.cookies.redirect || '/';
    res.clearCookie('redirect');
    res.redirect(redirect);
  });
};

exports.default = {

  use: function use(config, app, origin) {
    app.use((0, _cookieParser2.default)());
    app.use(_bodyParser2.default.json());
    app.use(_bodyParser2.default.urlencoded({ extended: false }));
    app.use((0, _expressSession2.default)({ secret: 'keyboard cat', resave: true, saveUninitialized: true }));
    var authenticators = (0, _keys2.default)(config);

    if (!authenticators.length) {
      return;
    }

    // passport setup Strategy
    _passport2.default.serializeUser(function (user, cb) {
      cb(null, user);
    });

    _passport2.default.deserializeUser(function (obj, cb) {
      cb(null, obj);
    });

    authenticators.forEach(function (authenticator) {
      applyStrategy(authenticator, config[authenticator], strategies[authenticator], origin);
    });

    app.use(_passport2.default.initialize());
    app.use(_passport2.default.session());

    // Endpoint to confirm authentication is still in valid
    app.get('/auth', function (req, res, next) {
      if (req.isAuthenticated()) {
        return next();
      }
      return res.status(401).json({});
    }, function (req, res) {
      res.status(200).json({
        id: req.user.id,
        token: req.user.token
      });
    });

    authenticators.forEach(function (authenticator) {
      applyEndpoint(app, authenticator);
    });
  }
};
module.exports = exports['default'];