'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _trim = require('babel-runtime/core-js/string/trim');

var _trim2 = _interopRequireDefault(_trim);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

exports.default = server;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _server = require('react-dom/server');

var _server2 = _interopRequireDefault(_server);

var _reactRouter = require('react-router');

var _reduxConnect = require('redux-connect');

var _createMemoryHistory = require('react-router/lib/createMemoryHistory');

var _createMemoryHistory2 = _interopRequireDefault(_createMemoryHistory);

var _reactRedux = require('react-redux');

var _recursiveReaddirSync = require('recursive-readdir-sync');

var _recursiveReaddirSync2 = _interopRequireDefault(_recursiveReaddirSync);

var _propertiesReader = require('properties-reader');

var _propertiesReader2 = _interopRequireDefault(_propertiesReader);

var _cookieDough = require('cookie-dough');

var _cookieDough2 = _interopRequireDefault(_cookieDough);

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _create = require('./create');

var _create2 = _interopRequireDefault(_create);

var _i18n = require('./i18n');

var _apiclient = require('./apiclient');

var _apiclient2 = _interopRequireDefault(_apiclient);

var _fetcher = require('./fetcher');

var _fetcher2 = _interopRequireDefault(_fetcher);

var _stringify3 = require('./stringify');

var _stringify4 = _interopRequireDefault(_stringify3);

var _createRouterOpts = require('./createRouterOpts');

var _createRouterOpts2 = _interopRequireDefault(_createRouterOpts);

var _Html = require('./Html');

var _Html2 = _interopRequireDefault(_Html);

var _App = require('./App');

var _App2 = _interopRequireDefault(_App);

var _Offline = require('./Offline');

var _Offline2 = _interopRequireDefault(_Offline);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var loadi18n = function loadi18n(dir, i18n) {
  var path = require('path');
  var files = (0, _recursiveReaddirSync2.default)(dir);
  files.map(function (file) {
    console.log('### loading lang files');
    console.log(file);
    var properties = (0, _propertiesReader2.default)(file);
    var lang = path.basename(file, '.properties');
    i18n[lang] = properties.getAllProperties();
  });
  console.log(i18n);
};

function server(_ref) {
  var app = _ref.app,
      path = _ref.path,
      urls = _ref.urls,
      origin = _ref.origin,
      i18ndir = _ref.i18ndir,
      reducers = _ref.reducers,
      routes = _ref.routes,
      handlers = _ref.handlers,
      statics = _ref.statics,
      _ref$isDevelopment = _ref.isDevelopment,
      isDevelopment = _ref$isDevelopment === undefined ? false : _ref$isDevelopment,
      _ref$manifest = _ref.manifest,
      manifest = _ref$manifest === undefined ? {} : _ref$manifest,
      _ref$colors = _ref.colors,
      colors = _ref$colors === undefined ? {} : _ref$colors;

  var i18n = {};
  loadi18n(i18ndir, i18n);
  if (_Offline2.default) {
    app.get('/offline', function (req, res) {
      res.send('<!doctype html>\n' + _server2.default.renderToString(_react2.default.createElement(_Html2.default, {
        assets: webpackIsomorphicTools.assets(),
        component: _react2.default.createElement(_Offline2.default, { colors: colors }),
        store: { getState: function getState() {} },
        statics: statics,
        enableScript: false
      })));
    });
  }
  var koikiSw = _fsExtra2.default.readFileSync(require('path').resolve(__dirname, './static/koiki-sw.js'));
  var cacheTargets = _fsExtra2.default.readJsonSync(require('path').resolve(process.cwd(), 'cache-targets.json'));
  app.use('/koiki-sw.js', function (req, res) {
    res.set('Content-Type', 'text/javascript');
    res.send('const CACHE_TARGETS = ' + (0, _stringify2.default)(cacheTargets) + ';' + koikiSw);
  });
  app.use('/', _express2.default.static(__dirname + '/static'));
  app.get('/manifest.json', function (req, res) {
    var lang = (0, _trim2.default)((req.headers['accept-language'] || '').split(',')[0].split('-')[0].split('_')[0]) || 'en';

    res.json((0, _assign2.default)({
      dir: 'ltr',
      lang: lang,
      name: origin,
      display: 'fullscreen',
      start_url: origin + '/' + lang,
      short_name: origin,
      theme_color: 'transparent',
      description: '',
      orientation: 'any',
      background_color: colors.background || 'transparent',
      related_applications: [],
      prefer_related_applications: false,
      icons: ['48', '72', '96', '144', '168', '192', '256', '384', '512'].map(function (size) {
        return {
          src: '/images/favicon.png',
          type: 'image/png',
          sizes: size + 'x' + size
        };
      })
    }, manifest));
  });

  app.get('*', function (req, res, next) {
    if (!isDevelopment && req.headers['x-forwarded-proto'] && req.headers['x-forwarded-proto'] !== 'https') {
      res.redirect(origin + req.url);
    } else {
      next();
    }
  });

  app.get('/', function (req, res) {
    var lang = (0, _trim2.default)((req.headers['accept-language'] || '').split(',')[0].split('-')[0].split('_')[0]) || 'en';
    console.log(lang, req.headers['accept-language']);
    res.redirect((0, _stringify4.default)(path, { lang: lang }));
  });

  app.use(path, function (req, res, next) {
    if (!i18n[req.params.lang]) {
      next();
      return;
    }
    if (isDevelopment) {
      // Do not cache webpack stats: the script file would change since
      // hot module replacement is enabled in the development env
      webpackIsomorphicTools.refresh();
    }
    var client = new _apiclient2.default({
      cookie: req.get('cookie'),
      origin: origin,
      referer: origin
    });
    var cookie = new _cookieDough2.default(req);
    var history = (0, _createMemoryHistory2.default)((0, _createRouterOpts2.default)());

    var store = (0, _create2.default)({ reducers: reducers, history: history, isDevelopment: isDevelopment });
    store.dispatch((0, _i18n.set)(i18n[req.params.lang]));

    var fetcher = new _fetcher2.default({
      client: client,
      dispatch: store.dispatch,
      urls: urls,
      type: 'server'
    });

    function hydrateOnClient() {
      res.send('<!doctype html>\n' + _server2.default.renderToString(_react2.default.createElement(_Html2.default, {
        assets: webpackIsomorphicTools.assets(),
        store: store,
        statics: statics
      })));
    }

    (0, _reactRouter.match)({
      history: history,
      routes: _react2.default.createElement(
        _reactRouter.Route,
        {
          urls: urls,
          origin: origin,
          component: _App2.default,
          cookie: cookie,
          fetcher: fetcher
        },
        routes(store, cookie)
      ),
      location: req.originalUrl
    }, function (error, redirectLocation, renderProps) {
      if (redirectLocation) {
        res.redirect(redirectLocation.pathname + redirectLocation.search);
      } else if (error) {
        handlers.error(error);
        res.set('Content-Type', 'text/html');
        res.status(500);
        hydrateOnClient();
      } else if (renderProps) {
        (0, _reduxConnect.loadOnServer)((0, _extends3.default)({}, renderProps, { store: store, cookie: cookie, helpers: { fetcher: fetcher } })).then(function () {
          var component = _react2.default.createElement(
            _reactRedux.Provider,
            { store: store, key: 'provider' },
            _react2.default.createElement(_reduxConnect.ReduxAsyncConnect, (0, _extends3.default)({}, renderProps, { store: store }))
          );
          res.set('Content-Type', 'text/html');
          res.status(200);

          global.navigator = { userAgent: req.headers['user-agent'] };
          res.send('<!doctype html>\n' + _server2.default.renderToString(_react2.default.createElement(_Html2.default, {
            assets: webpackIsomorphicTools.assets(),
            component: component,
            store: store,
            fetcher: fetcher.get(urls),
            statics: statics
          })));
        });
      } else {
        res.set('Content-Type', 'text/html');
        res.status(404).send('Not found');
      }
    });
  });
}
module.exports = exports['default'];